from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from sklearn.utils.estimator_checks import check_estimator

from sliced import SlicedInverseRegression, SlicedAverageVarianceEstimation


def test_sir():
    # a lot of warnings about changing n_slices for classification problems
    check_estimator(SlicedInverseRegression)


def test_save():
    # a lot of warnings about changing n_slices for classification problems
    check_estimator(SlicedAverageVarianceEstimation)



"""
============================================
Comparing PCA with Sliced Inverse Regression
============================================

A comparison of the subspace found by sliced inverse regression and
principal component analysis on the australian athletes dataset.
"""
import matplotlib.pyplot as plt

from sklearn.decomposition import PCA

from sliced.datasets import load_athletes
from sliced import SlicedInverseRegression

X, y = load_athletes()

# fit SIR model
sir = SlicedInverseRegression(n_slices=11).fit(X, y)
X_sir = sir.transform(X)

# fit PCA
pca = PCA(random_state=123).fit(X, y)
X_pca = pca.transform(X)

f, (ax1, ax2) = plt.subplots(1, 2, sharey=True)

ax1.scatter(X_sir[:, 0], y, c=y, cmap='viridis', linewidth=0.5, edgecolor='k')
ax1.set_title('SIR Subspace')
ax1.set_xlabel("$X\hat{\\beta}_{SIR}$")
ax1.set_ylabel("Lean Body Mass (kg)")

ax2.scatter(X_pca[:, 0], y, c=y, cmap='viridis', linewidth=0.5, edgecolor='k')
ax2.set_title('PCA Subspace')
ax2.set_xlabel("$X\hat{\\beta}_{PCA}$")
ax2.set_ylabel("Lean Body Mass (kg)")

plt.show()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import six
import warnings

import numpy as np
import scipy.linalg as linalg

from scipy import sparse
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.preprocessing import normalize

from .externals import check_array, check_X_y, check_is_fitted
from .base import slice_y, is_multioutput


__all__ = ['SlicedAverageVarianceEstimation']


class SlicedAverageVarianceEstimation(BaseEstimator, TransformerMixin):
    """Sliced Average Variance Estimation (SAVE) [1]

    Linear dimensionality reduction using the conditional covariance, Cov(X|y),
    to identify the directions defining the central subspace of the data.

    The algorithm performs a weighted principal component analysis on a
    transformation of slices of the covariance matrix of the whitened
    data, which has been sorted with respect to the target, y.

    Since SAVE looks at second moment information, it may miss first-moment
    information. In particular, it may miss linear trends. See
    :class:`sliced.sir.SlicedInverseRegression`, which is able to detect
    linear trends but may fail in other situations. If possible, both SIR and
    SAVE should be used when analyzing a dataset.

    Parameters
    ----------
    n_directions : int, str or None (default='auto')
        Number of directions to keep. Corresponds to the dimension of
        the central subpace. If n_directions=='auto', the number of directions
        is chosen by finding the maximum gap in the ordered eigenvalues of
        the var(X|y) matrix and choosing the directions before this gap.
        If n_directions==None, the number of directions equals the number of
        features.

    n_slices : int (default=10)
        The number of slices used when calculating the inverse regression
        curve. Truncated to at most the number of unique values of ``y``.

    copy : bool (default=True)
         If False, data passed to fit are overwritten and running
         fit(X).transform(X) will not yield the expected results,
         use fit_transform(X) instead.

    Attributes
    ----------
    directions_ : array, shape (n_directions, n_features)
        The directions in feature space, representing the
        central subspace which is sufficient to describe the conditional
        distribution y|X. The directions are sorted by ``eigenvalues_``.

    eigenvalues_ : array, shape (n_directions,)
        The eigenvalues corresponding to each of the selected directions.
        These are the eigenvalues of the covariance matrix
        of the inverse regression curve. Larger eigenvalues indicate
        more prevelant directions.

    Examples
    --------

    >>> import numpy as np
    >>> from sliced import SlicedAverageVarianceEstimation
    >>> from sliced.datasets import make_quadratic
    >>> X, y = make_quadratic(random_state=123)
    >>> save = SlicedAverageVarianceEstimation(n_directions=2)
    >>> save.fit(X, y)
    SlicedAverageVarianceEstimation(copy=True, n_directions=2, n_slices=10)
    >>> X_save = save.transform(X)

    References
    ----------

    [1] Shao, Y, Cook, RD and Weisberg, S (2007).
        "Marginal Tests with Sliced Average Variance Estimation",
        Biometrika, 94, 285-296.
    """
    def __init__(self, n_directions='auto', n_slices=10, copy=True):
        self.n_directions = n_directions
        self.n_slices = n_slices
        self.copy = copy

    def fit(self, X, y):
        """Fit the model with X and y.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            Training data, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape (n_samples,)
            The target values (class labels in classification, real numbers
            in regression).

        Returns
        -------
        self : object
            Returns the instance itself.
        """
        if sparse.issparse(X):
            raise TypeError("SlicedInverseRegression does not support "
                            "sparse input.")

        X, y = check_X_y(X, y, dtype=[np.float64, np.float32],
                         y_numeric=True, copy=self.copy)

        # handle n_directions == None
        if self.n_directions is None:
            n_directions = X.shape[1]
        elif (not isinstance(self.n_directions, six.string_types) and
                self.n_directions < 1):
            raise ValueError('The number of directions `n_directions` '
                             'must be >= 1. Got `n_directions`={}'.format(
                                self.n_directions))
        else:
            n_directions = self.n_directions

        # validate y
        if is_multioutput(y):
            raise TypeError("The target `y` cannot be multi-output.")

        n_samples, n_features = X.shape

        # Center and Whiten feature matrix using a QR decomposition
        # (this is the approach used in the dr package)
        if self.copy:
            X = X - np.mean(X, axis=0)
        else:
            X -= np.mean(X, axis=0)
        Q, R = linalg.qr(X, mode='economic')
        Z = np.sqrt(n_samples) * Q

        # sort rows of Z with respect to the target y
        Z = Z[np.argsort(y), :]

        # determine slices and counts
        slices, counts = slice_y(y, self.n_slices)
        self.n_slices_ = counts.shape[0]

        # construct slice covariance matrices
        M = np.zeros((n_features, n_features))
        for slice_idx in range(self.n_slices_):
            n_slice = counts[slice_idx]

            # center the entries in this slice
            Z_slice = Z[slices == slice_idx, :]
            Z_slice -= np.mean(Z_slice, axis=0)

            # slice covariance matrix
            V_slice = np.dot(Z_slice.T, Z_slice) / n_slice
            M_slice = np.eye(n_features) - V_slice
            M += (n_slice / n_samples) * np.dot(M_slice, M_slice)

        # eigen-decomposition of slice matrix
        evals, evecs = linalg.eigh(M)
        evecs = evecs[:, ::-1]
        evals = evals[::-1]
        try:
            # TODO: internally handle zero variance features. This would not
            # be a problem if we used svd, but does not match DR.
            directions = linalg.solve_triangular(np.sqrt(n_samples) * R, evecs)
        except (linalg.LinAlgError, TypeError):
            # NOTE: The TypeError is because of a bug in the reporting of scipy
            raise linalg.LinAlgError(
                "Unable to back-solve R for the dimension "
                "reducing directions. This is usually caused by the presents "
                "of zero variance features. Try removing these features with "
                "`sklearn.feature_selection.VarianceThreshold(threshold=0.)` "
                "and refitting.")

        # the number of directions is chosen by finding the maximum gap among
        # the ordered eigenvalues.
        if self.n_directions == 'auto':
            n_directions = np.argmax(np.abs(np.diff(evals))) + 1
        self.n_directions_ = n_directions

        # normalize directions
        directions = normalize(
            directions[:, :self.n_directions_], norm='l2', axis=0)
        self.directions_ = directions.T

        self.eigenvalues_ = evals[:self.n_directions_]

        return self

    def transform(self, X):
        """Apply dimension reduction on X.

        X is projected onto the EDR-directions previously extracted from a
        training set.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            New data, where n_samples in the number of samples
            and n_features is the number of features.


        Returns
        -------
        X_new : array-like, shape (n_samples, n_directions)

        """
        check_is_fitted(self, 'directions_')

        X = check_array(X)
        return np.dot(X, self.directions_.T)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np
import pytest

from sliced.datasets import load_athletes
from sliced.base import slice_y, grouped_sum


module_rng = np.random.RandomState(123)


def test_group_sum():
    """Simple group sum with a known result."""
    X = np.arange(25).reshape(5, 5)
    groups = np.array([0, 0, 1, 1, 2])

    expected_sum = np.array([
        [5, 7, 9, 11, 13],
        [25, 27, 29, 31, 33],
        [20, 21, 22, 23, 24]])
    np.testing.assert_array_equal(
        grouped_sum(X, groups),
        expected_sum
    )


def test_group_sum_all_same():
    """Test behavior when every row is in the same group."""
    X = np.arange(9).reshape(3, 3)
    groups = np.zeros(3)

    expected_sum = np.sum(X, axis=0).reshape(1, -1)
    np.testing.assert_array_equal(
        grouped_sum(X, groups),
        expected_sum
    )


def test_group_sum_all_different():
    """Test behavior when every row is in a different group."""
    X = np.arange(9).reshape(3, 3)
    groups = np.arange(3)
    np.testing.assert_array_equal(
        grouped_sum(X, groups),
        X
    )


def test_group_sum_too_many_groups():
    """Throw an error if too many groups are used."""
    X = np.arange(9).reshape(3, 3)
    groups = np.arange(4)
    with pytest.raises(IndexError):
        grouped_sum(X, groups)


def test_slice_raises_error_when_y_has_one_value():
    """Error if y only has one value."""
    y = np.ones(100)

    with pytest.raises(ValueError):
        slice_y(y)


def test_slice_y_simple():
    """Test a simple a target with 100 unique values and even slices."""
    shuffle = module_rng.permutation(100)
    y = np.arange(100)[shuffle]

    slice_indicator, counts = slice_y(y, n_slices=10)

    # all slices should have 10 counts
    np.testing.assert_array_equal(
        counts, np.repeat(10, 10))

    # continguous indices counting up from 10
    np.testing.assert_array_equal(
        slice_indicator,
        np.repeat(np.arange(10), 10)
    )


def test_uneven_slices():
    """Too match DR the slice function spills over to 6 slices if it cannot
    fit it into 5..."""
    shuffle = module_rng.permutation(68)
    y = np.arange(68)[shuffle]

    slice_indicator, counts = slice_y(y, n_slices=5)

    np.testing.assert_array_equal(
        counts,
        np.array([13, 13, 13, 13, 13, 3])
    )

    np.testing.assert_array_equal(
        slice_indicator,
        np.hstack((np.repeat(np.arange(5), 13), np.array([5, 5, 5])))
    )


def test_uneven_slices_within_y():
    """Make sure the slice function actually groups common y values together.
    """
    y = np.hstack((
        np.repeat(1, 13),
        np.repeat(2, 5),
        np.repeat(3, 25)))

    slice_indicator, counts = slice_y(y, n_slices=3)

    np.testing.assert_array_equal(
        counts, np.array([13, 5, 25]))

    expected_indicator = np.hstack((
        np.repeat(0, 13),
        np.repeat(1, 5),
        np.repeat(2, 25))
    )
    np.testing.assert_array_equal(
        slice_indicator, expected_indicator)


def test_remaining_obs_put_in_last_slice():
    """This was a bug found in testing. This particular dataset puts 9
    counts"""
    y = np.arange(100)

    slice_indicator, counts = slice_y(y, n_slices=11)

    np.testing.assert_array_equal(
        np.hstack((np.repeat(9, 10), 10)),
        counts
    )

    expected_slices = np.hstack((
        np.repeat(np.arange(10), 9),
        np.repeat(10, 10)
    ))
    np.testing.assert_array_equal(
        expected_slices,
        slice_indicator
    )


def test_slice_match_athletes():
    """Test that we match the slices used in the DR package on the
    athlete's dataset.
    """
    X, y = load_athletes()

    slice_indicator, counts = slice_y(y, n_slices=11)

    # counts per slice match DR package
    expected_counts = np.array([
        18, 18, 18, 18, 18, 19, 18, 19, 23, 18, 15])
    np.testing.assert_array_equal(counts, expected_counts)

    # slice indicators match DR package
    expected_indicator = np.array([
        6, 5, 4, 4, 2, 3, 5, 1, 3, 3, 8, 6, 1, 7, 5, 4, 3, 4, 6,
        4, 6, 6, 4, 3, 7, 6, 5, 4, 1, 2, 2, 5, 5, 6, 6, 2, 1, 3,
        2, 4, 4, 2, 3, 4, 2, 5, 4, 5, 3, 1, 3, 4, 3, 5, 5, 3, 3,
        3, 4, 2, 5, 5, 6, 5, 4, 2, 2, 2, 5, 8, 6, 6, 7, 5, 8, 1,
        3, 1, 3, 1, 3, 1, 2, 3, 2, 1, 2, 2, 2, 4, 4, 1, 2, 1, 2,
        1, 1, 1, 1, 1, 5, 8, 9, 10, 9, 8, 8, 9, 9, 10, 7, 9, 11, 9,
        9, 9, 11, 9, 10, 10, 1, 10, 10, 10, 10, 11, 10, 9, 11, 8, 10, 10, 11,
        11, 11, 9, 10, 8, 9, 8, 4, 7, 7, 8, 11, 10, 9, 6, 7, 8, 6, 5,
        4, 8, 3, 7, 7, 9, 9, 11, 9, 9, 11, 7, 9, 8, 7, 6, 6, 6, 7,
        7, 6, 6, 5, 11, 11, 11, 10, 7, 8, 9, 7, 9, 8, 8, 9, 7, 11, 9,
        9, 11, 8, 10, 10, 10, 7, 10, 8, 7, 6, 8,
    ])
    np.testing.assert_array_equal(
        slice_indicator, np.sort(expected_indicator) - 1)



"""
==================================
Sliced Average Variance Estimation
==================================

An example plot of :class:`sliced.save.SlicedAverageVarianceEstimation`
"""
import numpy as np
import matplotlib.pyplot as plt

from sliced import SlicedAverageVarianceEstimation
from sliced import datasets


X, y = datasets.make_quadratic(random_state=123)

save = SlicedAverageVarianceEstimation()
X_save = save.fit_transform(X, y)

# estimate of the first dimension reducing direction
beta1_hat = save.directions_[0, :]

plt.scatter(X_save[:, 0], y, c=y, cmap='viridis', linewidth=0.5, edgecolor='k')
plt.xlabel("$X\hat{\\beta_1}$")
plt.ylabel("y")

# annotation showing the direction found
beta_text = "$\\beta_1$ = " + "{0}".format([0.707, 0.707])
plt.annotate(beta_text, xy=(-1, 2))
beta1_hat_text = "$\hat{\\beta_1}$ = " + "{0}".format(
    np.round(beta1_hat, 3).tolist()[:2])
plt.annotate(beta1_hat_text, xy=(-1, 1.75))

plt.show()



import csv

import numpy as np

from os.path import join


def load_data(module_path, data_file_name, is_classification=True):
    with open(join(module_path, 'data', data_file_name)) as csv_file:
        data_file = csv.reader(csv_file)
        temp = next(data_file)
        n_samples = int(temp[0])
        n_features = int(temp[1])
        data = np.empty((n_samples, n_features))

        if is_classification:
            target_dtype = np.int
        else:
            target_dtype = np.float64
        target = np.empty((n_samples,), dtype=target_dtype)

        for i, ir in enumerate(data_file):
            data[i] = np.asarray(ir[:-1], dtype=np.float64)
            target[i] = np.asarray(ir[-1], dtype=target_dtype)

        return data, target



import csv
from os.path import dirname

from sliced.datasets.base import load_data


def load_athletes():
    """Load and return the australian athletes data set (regression).

    This dataset was collected in a study of how data on various
    characteristics of the blood varied with sport body size and sex of the
    athlete.

    Here the goal is to predict lean body mass (lbm) based on the logarithm
    of various attributes of the athelete.

    ===============     ===============
    Samples total                   202
    Dimensionality                    8
    Features             real, positive
    ===============     ===============

    Returns
    -------
    (data, target) : tuple
        The X feature matrix and the y target vector.

    References
    ----------

    [1] Telford, R.D. and Cunningham, R.B. 1991.
        "Sex, sport and body-size dependency of hematology in highly
        trained athletes", Medicine and Science in Sports and
        Exercise 23: 788-794.
    """
    module_path = dirname(__file__)
    data, target = load_data(module_path, 'athletes.csv',
                             is_classification=False)
    return data, target



"""
====================
Clustering with SAVE
====================

Sliced Average Variance Estimation is able to find three distinct clusters
in a dataset used to classify counterfeit swiss banknotes.
"""
import matplotlib.pyplot as plt

from sliced.datasets import load_banknote
from sliced import SlicedAverageVarianceEstimation


X, y = load_banknote()

save = SlicedAverageVarianceEstimation(n_directions=2, n_slices=2)
X_save = save.fit_transform(X, y)

plt.scatter(X_save[:, 0], X_save[:, 1], c=y, alpha=0.8, edgecolor='k')
plt.xlabel("$X\hat{\\beta}_{1}$")
plt.ylabel("$X\hat{\\beta}_{2}$")
plt.title("Swiss Banknote Data")

plt.show()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys

import numpy as np
import pytest

from scipy import sparse
from scipy import linalg
from sklearn.datasets import load_digits

from sliced import SlicedAverageVarianceEstimation
from sliced import datasets


def test_cubic():
    X, y = datasets.make_cubic(random_state=123)

    save = SlicedAverageVarianceEstimation().fit(X, y)

    true_beta = (1 / np.sqrt(2)) * np.hstack((np.ones(2), np.zeros(8)))
    angle = np.dot(true_beta, save.directions_[0, :])
    np.testing.assert_allclose(np.abs(angle), 1, rtol=1e-1)


def test_regression():
    """NOTE: subsequent calls may flip the direction of eigenvectors
        (mulitply by -1), so we can only compare absolute values.

        This was not a problem for svds.. investigate if we can get
        deterministic behavior back.
    """
    X, y = datasets.make_quadratic(random_state=123)

    for n_dir in range(1, X.shape[1]):
        save = SlicedAverageVarianceEstimation(n_directions=n_dir)

        # take shape is correct
        X_save = save.fit(X, y).transform(X)
        np.testing.assert_equal(X_save.shape[1], n_dir)

        # should match fit_transform
        X_save2 = save.fit_transform(X, y)
        np.testing.assert_allclose(np.abs(X_save), np.abs(X_save2))

        # call transform again and check if things are okay
        X_save = save.transform(X)
        X_save2 = save.fit_transform(X, y)
        np.testing.assert_allclose(np.abs(X_save), np.abs(X_save2))

        # there is one true angle it should fine
        true_beta = (1 / np.sqrt(2)) * np.hstack((np.ones(2), np.zeros(8)))
        angle = np.dot(true_beta, save.directions_[0, :])
        np.testing.assert_allclose(np.abs(angle), 1, rtol=1e-1)


def test_n_directions_none():
    X, y = datasets.make_cubic(random_state=123)
    sir = SlicedAverageVarianceEstimation(n_directions=None).fit(X, y)
    np.testing.assert_equal(sir.n_directions_, X.shape[1])


def test_n_slices_too_big():
    X = np.array([[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]],
                 dtype=np.float64)
    y = np.array([1, 1, 1, 0, 0, 0])

    save = SlicedAverageVarianceEstimation(n_directions=1, n_slices=10)
    save.fit(X, y)

    assert save.n_slices_ == 2


def test_single_y_value():
    rng = np.random.RandomState(123)

    X = rng.randn(100, 4)
    y = np.ones(100)

    with pytest.raises(ValueError):
        SlicedAverageVarianceEstimation().fit(X, y)


def test_sparse_not_supported():
    X, y = datasets.make_cubic(random_state=123)
    X = sparse.csr_matrix(X)

    with pytest.raises(TypeError):
        SlicedAverageVarianceEstimation().fit(X, y)


def test_n_directions_auto_heuristic():
    X, y = datasets.make_exponential(random_state=123)
    save = SlicedAverageVarianceEstimation(n_directions='auto').fit(X, y)
    assert save.n_directions_ == 2

    X_save = save.transform(X)
    assert X_save.shape == (500, 2)


def test_zero_variance_features():
    """Raise an informative error message when features of zero variance."""
    X, y = load_digits(return_X_y=True)

    with pytest.raises(linalg.LinAlgError):
        save = SlicedAverageVarianceEstimation(n_directions='auto').fit(X, y)


@pytest.mark.skipif(sys.platform == 'win32',
                    reason=("Lapack's eigh is not deterministic across ",
                            "platforms. The sign of some eigenvectors is ",
                            "flipped on win32."))
def test_matches_swiss_banknote():
    """Test that the results match the R dr package on a few common datasets.
    """
    X, y = datasets.load_banknote()
    save = SlicedAverageVarianceEstimation(n_directions=4).fit(X, y)

    np.testing.assert_allclose(
        save.eigenvalues_,
        np.array([0.87239404, 0.42288351, 0.12792117, 0.03771284])
    )

    expected_directions = np.array(
        [[0.03082069, 0.20309393, -0.25314643, -0.58931337, -0.56801632,
          0.47306135],
         [-0.2841728, -0.05472057, -0.15731808, 0.50606843, 0.33404888,
          0.72374622],
         [0.09905744, -0.88896348, 0.42252244, -0.00162151, -0.09222179,
          -0.11357311],
         [0.75251819, -0.26448055, 0.59669025, 0.03982343, -0.018666,
          0.07611073]],
    )
    np.testing.assert_allclose(
        save.directions_, expected_directions, atol=1e-8)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import six
import warnings

import numpy as np
import scipy.linalg as linalg

from scipy import sparse
from sklearn.base import BaseEstimator, TransformerMixin
from sklearn.preprocessing import normalize

from .externals import check_array, check_X_y, check_is_fitted
from .base import slice_y, grouped_sum, is_multioutput


__all__ = ['SlicedInverseRegression']


class SlicedInverseRegression(BaseEstimator, TransformerMixin):
    """Sliced Inverse Regression (SIR) [1]

    Linear dimensionality reduction using the inverse regression curve,
    E[X|y], to identify the directions defining the central subspace of
    the data.

    The inverse comes from the fact that X and y are reversed with respect
    to the standard regression framework (estimating E[y|X]).

    The algorithm performs a weighted principal component analysis on
    slices of the whitened data, which has been sorted with respect to
    the target, y.

    For a binary target the directions found correspond to those found
    with Fisher's Linear Discriminant Analysis (LDA).

    Note that SIR may fail to estimate the directions if the conditional
    density X|y is symmetric, so that E[X|y] = 0. See
    :class:`sliced.save.SlicedAverageVarianceEstimation`,
    which is able to overcome this limitation but may fail to pick up on
    linear trends. If possible, both SIR and SAVE should be used when analyzing
    a dataset.

    Parameters
    ----------
    n_directions : int, str or None (default='auto')
        Number of directions to keep. Corresponds to the dimension of
        the central subpace. If n_directions=='auto', the number of directions
        is chosen by finding the maximum gap in the ordered eigenvalues of
        the var(X|y) matrix and choosing the directions before this gap.
        If n_directions==None, the number of directions equals the number of
        features.

    n_slices : int (default=10)
        The number of slices used when calculating the inverse regression
        curve. Truncated to at most the number of unique values of ``y``.

    copy : bool (default=True)
         If False, data passed to fit are overwritten and running
         fit(X).transform(X) will not yield the expected results,
         use fit_transform(X) instead.

    Attributes
    ----------
    directions_ : array, shape (n_directions, n_features)
        The directions in feature space, representing the
        central subspace which is sufficient to describe the conditional
        distribution y|X. The directions are sorted by ``eigenvalues_``.

    eigenvalues_ : array, shape (n_directions,)
        The eigenvalues corresponding to each of the selected directions.
        These are the eigenvalues of the covariance matrix
        of the inverse regression curve. Larger eigenvalues indicate
        more prevelant directions.

    Examples
    --------

    >>> import numpy as np
    >>> from sliced import SlicedInverseRegression
    >>> from sliced.datasets import make_cubic
    >>> X, y = make_cubic(random_state=123)
    >>> sir = SlicedInverseRegression(n_directions=2)
    >>> sir.fit(X, y)
    SlicedInverseRegression(copy=True, n_directions=2, n_slices=10)
    >>> X_sir = sir.transform(X)

    References
    ----------

    [1] Li, K C. (1991)
        "Sliced Inverse Regression for Dimension Reduction (with discussion)",
        Journal of the American Statistical Association, 86, 316-342.
    """
    def __init__(self, n_directions='auto', n_slices=10, copy=True):
        self.n_directions = n_directions
        self.n_slices = n_slices
        self.copy = copy

    def fit(self, X, y):
        """Fit the model with X and y.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            Training data, where n_samples is the number of samples
            and n_features is the number of features.

        y : array-like, shape (n_samples,)
            The target values (class labels in classification, real numbers
            in regression).

        Returns
        -------
        self : object
            Returns the instance itself.
        """
        if sparse.issparse(X):
            raise TypeError("SlicedInverseRegression does not support "
                            "sparse input.")

        X, y = check_X_y(X, y, dtype=[np.float64, np.float32],
                         accept_sparse=['csr'],
                         y_numeric=True, copy=self.copy)

        if self.n_directions is None:
            n_directions = X.shape[1]
        elif (not isinstance(self.n_directions, six.string_types) and
                self.n_directions < 1):
            raise ValueError('The number of directions `n_directions` '
                             'must be >= 1. Got `n_directions`={}'.format(
                                self.n_directions))
        else:
            n_directions = self.n_directions

        # validate y
        if is_multioutput(y):
            raise TypeError("The target `y` cannot be multi-output.")

        n_samples, n_features = X.shape

        # Center and Whiten feature matrix using a QR decomposition
        # (this is the approach used in the dr package)
        if self.copy:
            X = X - np.mean(X, axis=0)
        else:
            X -= np.mean(X, axis=0)
        Q, R = linalg.qr(X, mode='economic')
        Z = np.sqrt(n_samples) * Q
        Z = Z[np.argsort(y), :]

        # determine slice indices and counts per slice
        slices, counts = slice_y(y, self.n_slices)
        self.n_slices_ = counts.shape[0]

        # means in each slice (sqrt factor takes care of the weighting)
        Z_means = grouped_sum(Z, slices) / np.sqrt(counts.reshape(-1, 1))
        M = np.dot(Z_means.T, Z_means) / n_samples

        # eigen-decomposition of slice matrix
        evals, evecs = linalg.eigh(M)
        evecs = evecs[:, ::-1]
        evals = evals[::-1]
        try:
            # TODO: internally handle zero variance features. This would not
            # be a problem if we used svd, but does not match DR.
            directions = linalg.solve_triangular(np.sqrt(n_samples) * R, evecs)
        except (linalg.LinAlgError, TypeError):
            # NOTE: The TypeError is because of a bug in the reporting of scipy
            raise linalg.LinAlgError(
                "Unable to back-solve R for the dimension "
                "reducing directions. This is usually caused by the presents "
                "of zero variance features. Try removing these features with "
                "`sklearn.feature_selection.VarianceThreshold(threshold=0.)` "
                "and refitting.")

        # the number of directions is chosen by finding the maximum gap among
        # the ordered eigenvalues.
        if self.n_directions == 'auto':
            n_directions = np.argmax(np.abs(np.diff(evals))) + 1
        self.n_directions_ = n_directions

        directions = normalize(
            directions[:, :self.n_directions_], norm='l2', axis=0)
        self.directions_ = directions.T
        self.eigenvalues_ = evals[:self.n_directions_]

        return self

    def transform(self, X):
        """Apply dimension reduction on X.

        X is projected onto the central subspace  previously extracted from a
        training set.

        Parameters
        ----------
        X : array-like, shape (n_samples, n_features)
            New data, where n_samples in the number of samples
            and n_features is the number of features.


        Returns
        -------
        X_new : array-like, shape (n_samples, n_directions)

        """
        check_is_fitted(self, 'directions_')

        X = check_array(X)
        return np.dot(X, self.directions_.T)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np

from sklearn.utils import check_random_state


__all__ = ['make_cubic', 'make_quadratic', 'make_polynomial',
           'make_exponential']


def make_cubic(n_samples=500, n_features=10, n_informative=2,
               random_state=None):
    """Generates a dataset with a cubic response curve.

    Inputs X are independent normally distributed features. The output y
    is created according to the formula::

        beta = np.hstack((
            np.ones(n_informative), np.zeros(n_features - n_informative)))
        h = np.dot(X, beta)
        y(h) = 0.125 * h ** 3 + 0.5 * N(0, 1).

    Out of the `n_features` features,  only `n_informative` are actually
    used to compute `y`. The remaining features are independent of `y`.
    As such the central subspace is one dimensional and consists of the
    `h` axis.

    Parameters
    ----------
    n_samples : int, optimal (default=500)
        The number of samples.

    n_features : int, optional (default=10)
        The number of features. Should be at least equal to `n_informative`.

    n_informative : int, optional (default=2)
        The number of informative features used to construct h. Should be
        at least 1.

    random_state : int, RandomState instance or None, optional (default=None)
        If int, random_state is the seed used by the random number generator;
        If RandomState instance, random_state is the random number generator;
        If None, the random number generator is the RandomState instance used
        by `np.random`.

    Returns
    -------
    X : array of shape [n_samples, n_features]
        The input samples.

    y : array of shape [n_samples]
        The output values.
    """
    rng = check_random_state(random_state)

    if n_informative < 1:
        raise ValueError("`n_informative` must be >= 1. "
                         "Got n_informative={0}".format(n_informative))

    if n_features < n_informative:
        raise ValueError("`n_features` must be >= `n_informative`. "
                         "Got n_features={0} and n_informative={1}".format(
                            n_features, n_informative))

    # normally distributed features
    X = rng.randn(n_samples, n_features)

    # beta is a linear combination of informative features
    beta = np.hstack((
        np.ones(n_informative), np.zeros(n_features - n_informative)))

    # cubic in subspace
    y = 0.125 * np.dot(X, beta) ** 3
    y += 0.5 * rng.randn(n_samples)

    return X, y


def make_quadratic(n_samples=500, n_features=10, n_informative=2,
                   random_state=None):
    """Generates a dataset with a quadratic response curve.

    Inputs X are independent normally distributed features. The output y
    is created according to the formula::

        beta = np.hstack((
            np.ones(n_informative), np.zeros(n_features - n_informative)))
        h = np.dot(X, beta)
        y(h) = 0.125 * h ** 2 + 0.5 * N(0, 1).

    Out of the `n_features` features,  only `n_informative` are actually
    used to compute `y`. The remaining features are independent of `y`.
    As such the central subspace is one dimensional and consists of the
    `h` axis.

    Parameters
    ----------
    n_samples : int, optimal (default=500)
        The number of samples.

    n_features : int, optional (default=10)
        The number of features. Should be at least equal to `n_informative`.

    n_informative : int, optional (default=2)
        The number of informative features used to construct h. Should be
        at least 1.

    random_state : int, RandomState instance or None, optional (default=None)
        If int, random_state is the seed used by the random number generator;
        If RandomState instance, random_state is the random number generator;
        If None, the random number generator is the RandomState instance used
        by `np.random`.

    Returns
    -------
    X : array of shape [n_samples, n_features]
        The input samples.

    y : array of shape [n_samples]
        The output values.
    """
    rng = check_random_state(random_state)

    if n_informative < 1:
        raise ValueError("`n_informative` must be >= 1. "
                         "Got n_informative={0}".format(n_informative))

    if n_features < n_informative:
        raise ValueError("`n_features` must be >= `n_informative`. "
                         "Got n_features={0} and n_informative={1}".format(
                            n_features, n_informative))

    # normally distributed features
    X = rng.randn(n_samples, n_features)

    # beta is a linear combination of informative features
    beta = np.hstack((
        np.ones(n_informative), np.zeros(n_features - n_informative)))

    # cubic in subspace
    y = 0.125 * np.dot(X, beta) ** 2
    y += 0.25 * rng.randn(n_samples)

    return X, y


def make_polynomial(n_samples=500, n_features=10, random_state=None):
    """Generates a dataset with a polynomial response curve that combines
    a quadratic and cubic response. There are two primary dimensions instead
    of one.

    Inputs X are independent normally distributed features. The output y
    is created according to the formula::

        beta1 = np.hstack((
            [1, 1, 1], np.zeros(n_features - 3)))
        beta2 = np.hstack((
            [1, -1, -1], np.zeros(n_features - 3)))

        u = np.dot(X, beta1)
        v = np.dot(X, beta2)
        y(u, v) = u + u ** 3 + v ** 2 + N(0, 1)

    Out of the `n_features` features,  only 3 are actually
    used to compute `y`. The remaining features are independent of `y`.
    As such the central subspace is two dimensional and consists of the
    `u` and `v` axes.

    Parameters
    ----------
    n_samples : int, optimal (default=500)
        The number of samples.

    n_features : int, optional (default=10)
        The number of features. Should be at least equal to 3, the number
        of informative features.

    random_state : int, RandomState instance or None, optional (default=None)
        If int, random_state is the seed used by the random number generator;
        If RandomState instance, random_state is the random number generator;
        If None, the random number generator is the RandomState instance used
        by `np.random`.

    Returns
    -------
    X : array of shape [n_samples, n_features]
        The input samples.

    y : array of shape [n_samples]
        The output values.
    """
    rng = check_random_state(random_state)

    if n_features < 3:
        raise ValueError("`n_features` must be >= 3. "
                         "Got n_features={0}".format(n_features))

    # normally distributed features
    X = rng.randn(n_samples, n_features)

    # beta is a linear combination of informative features
    beta1 = np.hstack((
        [1, 1, 1], np.zeros(n_features - 3)))
    beta2 = np.hstack((
        [1, -1, -1], np.zeros(n_features - 3)))

    u = np.dot(X, beta1)
    v = np.dot(X, beta2)
    y = 0.7 * (u + u ** 3) + v ** 2
    y += 0.5 * rng.randn(n_samples)

    return X, y


def make_exponential(n_samples=500, n_features=20, random_state=None):
    """Generates a dataset with a exponential response curve.

    Inputs X are independent normally distributed features. The output y
    is created according to the formula::

        beta1 = np.hstack((
            np.ones(7), np.zeros(n_features - 7)))
        beta2 = np.hstack((
            np.zeros(7), np.ones(4), np.zeros(n_features - 11)))
        u = np.dot(X, beta1)
        v = np.dot(X, beta2)
        y(u, v) = u * np.exp(v) + N(0, 1)

    Parameters
    ----------
    n_samples : int, optimal (default=500)
        The number of samples.

    n_features : int, optional (default=10)
        The number of features. Should be at least equal to `n_informative`.

    random_state : int, RandomState instance or None, optional (default=None)
        If int, random_state is the seed used by the random number generator;
        If RandomState instance, random_state is the random number generator;
        If None, the random number generator is the RandomState instance used
        by `np.random`.

    Returns
    -------
    X : array of shape [n_samples, n_features]
        The input samples.

    y : array of shape [n_samples]
        The output values.
    """
    rng = check_random_state(random_state)

    if n_features < 11:
        raise ValueError("`n_features` must be >= 11. "
                         "Got n_features={0}".format(n_features))

    X = rng.randn(n_samples, 20)

    beta1 = np.hstack((
        np.ones(7), np.zeros(n_features - 7)))
    beta2 = np.hstack((
        np.zeros(7), np.ones(4), np.zeros(n_features - 11)))

    u = np.dot(X, beta1)
    v = np.dot(X, beta2)
    y = u * np.exp(v)
    y += rng.randn(n_samples)

    return X, y



import csv
from os.path import dirname

from sliced.datasets.base import load_data


def load_banknote():
    """Load and return the swiss banknote dataset (classification).

    Six measurements made on 100 genuine Swiss banknotes and 100 counterfeit
    ones.

    Features
    ========
    Length:
        Length of bill (mm)
    Left:
        Width of left edge (mm)
    Right:
        Width of right edge (mm)
    Bottom:
        Bottom margin width (mm)
    Top:
        Top margin width (mm)
    Diagonal:
        Length of image diagonal (mm)
    Y:
        0 = genuine, 1 = conterfeit

    =================   =================
    Classes                             2
    Samples per class    100 (Y), 100 (N)
    Samples total                     200
    Dimensionality                      6
    Features               real, positive
    =================   =================
    """
    module_path = dirname(__file__)
    data, target = load_data(module_path, 'banknote.csv')
    return data, target



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from sliced import datasets
from sliced import base
from sliced import externals

from sliced.sir import *
from sliced.save import *
from sliced.version import *



"""
=======================
Binary Targets with SIR
=======================

Sliced Inverse Regression is able to find a one-dimensional subspace
that seperates cases in the famous breast cancer dataset.
"""
import matplotlib.pyplot as plt

from sklearn.datasets import load_breast_cancer
from sliced import SlicedInverseRegression

X, y = load_breast_cancer(return_X_y=True)

sir = SlicedInverseRegression(n_directions=2).fit(X, y)
X_sir = sir.transform(X)

plt.scatter(X_sir[:, 0], X_sir[:, 1], c=y, alpha=0.8, edgecolor='k')
plt.xlabel("$X\hat{\\beta}_{1}$")
plt.ylabel("$X\hat{\\beta}_{2}$")
plt.title("Breast Cancer Data")

plt.show()



from __future__ import print_function

import os
import sys

from setuptools import setup, find_packages


HERE = os.path.dirname(os.path.abspath(__file__))

# import ``__version__` from code base
exec(open(os.path.join(HERE, 'sliced', 'version.py')).read())


with open('requirements.txt') as f:
    INSTALL_REQUIRES = [l.strip() for l in f.readlines() if l]


with open('test_requirements.txt') as f:
    TEST_REQUIRES = [l.strip() for l in f.readlines() if l]


try:
    import numpy
except ImportError:
    print('numpy is required during installation')
    sys.exit(1)


try:
    import scipy
except ImportError:
    print('scipy is required during installation')
    sys.exit(1)


DISTNAME = 'sliced'
DESCRIPTION = 'Toolbox for sufficient dimension reduction (SDR).'
with open('README.rst') as f:
    LONG_DESCRIPTION = f.read()
MAINTAINER = 'Joshua D. Loyal'
MAINTAINER_EMAIL = 'jloyal25@gmail.com'
URL ='https://joshloyal.github.io/sliced'
DOWNLOAD_URL = 'https://pypi.org/project/sliced/#files'
LICENSE = 'MIT'
VERSION = __version__
CLASSIFIERS = ['Intended Audience :: Science/Research',
               'Intended Audience :: Developers',
               'License :: OSI Approved :: MIT License',
               'Programming Language :: Python',
               'Topic :: Software Development',
               'Topic :: Scientific/Engineering',
               'Operating System :: Microsoft :: Windows',
               'Operating System :: POSIX',
               'Operating System :: Unix',
               'Operating System :: MacOS',
               'Programming Language :: Python :: 2',
               'Programming Language :: Python :: 2.7',
               'Programming Language :: Python :: 3',
               'Programming Language :: Python :: 3.5',
               'Programming Language :: Python :: 3.6',
              ]


setup(
    name=DISTNAME,
    maintainer=MAINTAINER,
    maintainer_email=MAINTAINER_EMAIL,
    description=DESCRIPTION,
    license=LICENSE,
    url=URL,
    version=VERSION,
    download_url=DOWNLOAD_URL,
    long_description=LONG_DESCRIPTION,
    zip_safe=False,
    classifiers=CLASSIFIERS,
    include_package_data=True,
    packages=find_packages(),
    install_requires=INSTALL_REQUIRES,
    extras_require={'test': TEST_REQUIRES},
    setup_requires=['pytest-runner'],
    tests_require=TEST_REQUIRES,
)



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from pkg_resources import parse_version

import warnings

import numpy as np
import scipy.linalg as linalg


NUMPY_UNIQUE_COUNTS_VERSION = '1.9.0'


def is_multioutput(y):
    """Whether the target y is multi-output (or multi-index)"""
    return hasattr(y, "shape") and y.ndim == 2 and y.shape[1] > 1


def grouped_sum(array, groups):
    """Sums an array by groups. Groups are assumed to be contiguous by row."""
    inv_idx = np.concatenate(([0], np.diff(groups).nonzero()[0] + 1))
    return np.add.reduceat(array, inv_idx)


def unique_counts(arr):
    """Determine the unique values and the number of times they occur in a one
    dimensional array.

    This is a wrapper around numpy's unique function.
    In order to keep the numpy dependency below 1.9 this function falls
    back to the slow version of getting the unique counts array by counting
    the indices of the inverse array.

    Parameters
    ----------
    arr : array_like
        Input array. This array will be flattened if it is not already 1-D.

    Returns
    -------
    unique : ndarray
        The sorted unique values.

    unique_counts : ndarray
        The number of times each of the unique values compes up in the orginal
        array.
    """
    if (parse_version(np.__version__) >=
            parse_version(NUMPY_UNIQUE_COUNTS_VERSION)):
        unique, counts = np.unique(arr, return_counts=True)
    else:
        unique, unique_inverse = np.unique(arr, return_inverse=True)
        counts = np.bincount(unique_inverse)
    return unique, counts


def slice_y(y, n_slices=10):
    """Determine non-overlapping slices based on the target variable, y.

    Parameters
    ----------
    y : array_like, shape (n_samples,)
        The target values (class labels in classification, real numbers
        in regression).

    n_slices : int (default=10)
        The number of slices used when calculating the inverse regression
        curve. Truncated to at most the number of unique values of ``y``.

    Returns
    -------
    slice_indicator : ndarray, shape (n_samples,)
        Index of the slice (from 0 to n_slices) that contains this
        observation.
    slice_counts :  ndarray, shape (n_slices,)
        The number of counts in each slice.
    """
    unique_y_vals, counts = unique_counts(y)
    cumsum_y = np.cumsum(counts)

    # `n_slices` must be less-than or equal to the number of unique values
    # of `y`.
    n_y_values = unique_y_vals.shape[0]
    if n_y_values == 1:
        raise ValueError("The target only has one unique y value. It does "
                         "not make sense to fit SIR or SAVE in this case.")
    elif n_slices >= n_y_values:
        if n_slices > n_y_values:
            warnings.warn(
                "n_slices greater than the number of unique y values. "
                "Setting n_slices equal to {0}.".format(counts.shape[0]))
        # each y value gets its own slice. usually the case for classification
        slice_partition = np.hstack((0, cumsum_y))
    else:
        # attempt to put this many observations in each slice.
        # not always possible since we need to group common y values together
        # NOTE: This should be ceil, but this package is attempting to
        #       replicate the slices used by R's DR package which uses floor.
        n_obs = np.floor(y.shape[0] / n_slices)

        # Loop through the unique y value sums and group
        # slices together with the goal of 2 <= # in slice <= n_obs
        # Find index in y unique where slice begins and ends
        n_samples_seen = 0
        slice_partition = [0]  # index in y of start of a new slice
        while n_samples_seen < y.shape[0] - 2:
            slice_start = np.where(cumsum_y >= n_samples_seen + n_obs)[0]
            if slice_start.shape[0] == 0:  # this means we've reached the end
                slice_start = cumsum_y.shape[0] - 1
            else:
                slice_start = slice_start[0]

            n_samples_seen = cumsum_y[slice_start]
            slice_partition.append(n_samples_seen)

    # turn partitions into an indicator
    slice_indicator = np.ones(y.shape[0], dtype=np.int)
    for j, (start_idx, end_idx) in enumerate(
            zip(slice_partition, slice_partition[1:])):

        # this just puts any remaining observations in the last slice
        if j == len(slice_partition) - 2:
            slice_indicator[start_idx:] = j
        else:
            slice_indicator[start_idx:end_idx] = j

    slice_counts = np.bincount(slice_indicator)
    return slice_indicator, slice_counts



"""
=========================
Sliced Inverse Regression
=========================

An example plot of :class:`sliced.sir.SlicedInverseRegression`
"""
import numpy as np
import matplotlib.pyplot as plt

from sliced import SlicedInverseRegression
from sliced import datasets


X, y = datasets.make_cubic(random_state=123)

sir = SlicedInverseRegression()
X_sir = sir.fit_transform(X, y)

# estimate of the first dimension reducing directions
beta1_hat = sir.directions_[0, :]


# plot data projected onto the first direction
plt.scatter(X_sir[:, 0], y, c=y, cmap='viridis', linewidth=0.5, edgecolor='k')
plt.xlabel("$X\hat{\\beta_1}$")
plt.ylabel("y")

# annotation showing the direction found
beta_text = "$\\beta_1$ = " + "{0}".format([0.707, 0.707])
plt.annotate(beta_text, xy=(-2, 6.5))
beta1_hat_text = "$\hat{\\beta_1}$ = " + "{0}".format(
    np.round(beta1_hat, 3).tolist()[:2])
plt.annotate(beta1_hat_text, xy=(-2, 7.5))

plt.show()



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys

import numpy as np
import pytest

from scipy import sparse
from scipy import linalg
from sklearn.datasets import load_digits
from sklearn.discriminant_analysis import LinearDiscriminantAnalysis

from sliced import SlicedInverseRegression
from sliced import datasets


def test_regression():
    """NOTE: subsequent calls may flip the direction of eigenvectors
        (mulitply by -1), so we can only compare absolute values.

        This was not a problem for svds.. investigate if we can get
        deterministic behavior back.
    """
    X, y = datasets.make_cubic(random_state=123)

    for n_dir in range(1, X.shape[1]):
        sir = SlicedInverseRegression(n_directions=n_dir)

        # take shape is correct
        X_sir = sir.fit(X, y).transform(X)
        np.testing.assert_equal(X_sir.shape[1], n_dir)

        # should match fit_transform
        X_sir2 = sir.fit_transform(X, y)
        np.testing.assert_allclose(np.abs(X_sir), np.abs(X_sir2))

        # call transform again and check if things are okay
        X_sir = sir.transform(X)
        X_sir2 = sir.fit_transform(X, y)
        np.testing.assert_allclose(np.abs(X_sir), np.abs(X_sir2))

        # there is one true angle it should fine
        true_beta = (1 / np.sqrt(2)) * np.hstack((np.ones(2), np.zeros(8)))
        angle = np.dot(true_beta, sir.directions_[0, :])
        np.testing.assert_allclose(np.abs(angle), 1, rtol=1e-1)


def test_classification():
    """SIR is LDA for classification so lets test some predictions."""
    # Data is just 6 separable points in the plane
    X = np.array([[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]],
                 dtype=np.float64)
    y = np.array([1, 1, 1, 0, 0, 0])

    sir = SlicedInverseRegression(n_directions=1, n_slices=2).fit(X, y)
    lda = LinearDiscriminantAnalysis(solver='eigen').fit(X, y)

    y_pred = sir.transform(X) > 0
    np.testing.assert_equal(y, y_pred.ravel())
    np.testing.assert_equal(lda.predict(X), y_pred.ravel())


def test_n_slices_too_big():
    X = np.array([[-2, -1], [-1, -1], [-1, -2], [1, 1], [1, 2], [2, 1]],
                 dtype=np.float64)
    y = np.array([1, 1, 1, 0, 0, 0])

    sir = SlicedInverseRegression(n_directions=1, n_slices=10).fit(X, y)

    assert sir.n_slices_ == 2


def test_n_directions_none():
    X, y = datasets.make_cubic(random_state=123)
    sir = SlicedInverseRegression(n_directions=None).fit(X, y)
    np.testing.assert_equal(sir.n_directions_, X.shape[1])


def test_single_y_value():
    rng = np.random.RandomState(123)

    X = rng.randn(100, 4)
    y = np.ones(100)

    with pytest.raises(ValueError):
        SlicedInverseRegression().fit(X, y)


def test_sparse_not_supported():
    X, y = datasets.make_cubic(random_state=123)
    X = sparse.csr_matrix(X)
    with pytest.raises(TypeError):
        SlicedInverseRegression().fit(X, y)


def test_n_directions_auto_heuristic():
    X, y = datasets.make_exponential(random_state=123)
    sir = SlicedInverseRegression(n_directions='auto').fit(X, y)
    assert sir.n_directions_ == 2

    X_sir = sir.transform(X)
    assert X_sir.shape == (500, 2)


def test_zero_variance_features():
    """Raise an informative error message when features of zero variance."""
    X, y = load_digits(return_X_y=True)

    with pytest.raises(linalg.LinAlgError):
        sir = SlicedInverseRegression(n_directions='auto').fit(X, y)


@pytest.mark.skipif(sys.platform == 'win32',
                    reason=("Lapack's eigh is not deterministic across ",
                            "platforms. The sign of some eigenvectors is ",
                            "flipped on win32."))
def test_matches_athletes():
    """Test that the resutls match the R dr package on a ais dataset.
    """
    X, y = datasets.load_athletes()
    sir = SlicedInverseRegression(n_directions=4, n_slices=11).fit(X, y)

    np.testing.assert_allclose(
        sir.eigenvalues_,
        np.array([0.957661631, 0.245041613, 0.107075941, 0.090413047])
    )

    expected_directions = np.array(
        [[1.50963358e-01,  -9.16480522e-01,  -1.31538894e-01,  -9.33588596e-02,
          4.46783829e-03,  -1.88973540e-01,   2.74758965e-01,  -5.63123794e-03],
         [-5.01785457e-02,  -1.94229862e-01,   6.85475076e-01,  -4.33408964e-02,
          1.83380846e-04,   3.47565293e-01,  -6.05830142e-01,   1.30588502e-02],
         [1.08983356e-01,  -2.01236965e-01,   7.19975455e-01,   4.64453982e-01,
          4.49759016e-02,   2.94969081e-01,  -3.41966152e-01,  -8.70270913e-02],
         [-2.21020634e-03,  -8.97220257e-02,  -6.63097774e-01,   2.90838658e-01,
          7.19045566e-02,   3.70563626e-02,   6.78877114e-01,   1.55472144e-02]]
    )

    np.testing.assert_allclose(
        sir.directions_, expected_directions)
